# A token that is created by the lexer and can be recognized by the parser.
#
# @author Paul T.
class Calcula::Token

  # @return [Symbol] Returns the type of the token
  attr_reader :id

  # @return [String] Returns the string segment from the source
  attr_reader :text

  # Constructs a new Token that can be used with `Calcula::Parser`
  #
  # @param id [symbol] The type of the Token
  # @param text [string] The actual text
  def initialize(id, text)
    @id = id
    @text = text
  end

  # Converts the token into a format of `[:id => :text]`
  #
  # @return [String] The string representation of the Token in above format
  def to_s
    return "[#{@id}=>#{text}]"
  end

  # Duplicates a token
  #
  # @return [Calcula::Token] A duplicate of the token
  def dup
    Token.new(@id, @text)
  end
end
